#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>

#include "config.h"
#include "q2-wrapper.h"

struct commandline line;
struct directories dir;

int	quit_verbosity;

void greetings(void)
/*
 * Spew some greetings
 *
 */
{
#ifndef _DEBUG_
	printf("%s version %s.\n", NAME, VERSION);
	printf("Copyright %s by %s <%s>\n", YEAR, AUTHOR, EMAIL);
	printf("Distributed under the GNU General Public Licence version 2.\n");
	printf("This software is NOT supported by Id Software or ");	
	printf("Activision.\n\n");
#endif
}

/************************************************************************/

void die(void)
{
#ifndef _DEBUG_
	if (quit_verbosity == BIG_QUIT)
		printf("\n");
		greetings();
#endif
	if (quit_verbosity > TERSE_QUIT) 
		printf("==== Q2-wrapper exiting ====\n\n");
}

/************************************************************************/

void usage(char *progname)
{
	printf("\nUsage: %s [-q] | [-v] | [-h]\n", progname);
	printf("-q  Quiet.  Don't output Quake2 console message or\n");
	printf("    q2-wrapper messages to standard output\n");
	printf("-v  Verbose.  Output Quake2 console messsage and\n");
	printf("    q2-wrapper messages.\n");
	printf("-h  Help.  Display this help message.\n\n");
	printf("\"man q2-wrapper\" for more information\n\n");
	quit_verbosity = TERSE_QUIT;
	exit(0);
}

/************************************************************************/

void makeline (char lineout[LINELEN])
/*
 * Convert the command line struct into the actual string passed to the
 * quake2 binary.
 *
 */
{
	lineout[0] = 0;
	strcat(lineout, "+set vid_ref ");
	strncat(lineout, line.vid_ref, RENDLEN);
	strcat(lineout, " +set basedir ");
	strncat(lineout, line.basedir, PATHLEN);
	strcat(lineout, " +set cddir ");
	strncat(lineout, line.cddir, PATHLEN);
	strcat(lineout, " +set cd_dev ");
	strncat(lineout, line.cd_dev, DEV_PATHLEN);
}

/************************************************************************/

int checkdisplay(void)
{
	int retval = REF_SOFTX;

	if (getenv("DISPLAY") == NULL) {
		if (USE_GL == FALSE) {
			strncpy(line.vid_ref, SVGALIB_REND_SOFT, RENDLEN);
			retval = REF_SOFT;
		}
		else {
			strncpy(line.vid_ref, SVGALIB_REND_GL, RENDLEN);
			retval = REF_GL;
		}
	}
	else {
		if (USE_GL == FALSE) {
			strncpy(line.vid_ref, X11_REND_SOFT, RENDLEN);
			retval = REF_SOFTX;
		}
		else {
			strncpy(line.vid_ref, X11_REND_GL, RENDLEN);
			retval = REF_GLX;
		}
	}
	return(retval);
}

/************************************************************************/

int getpaths(void)
{
	char 	*home;

	if ((home = getenv("HOME")) == NULL) {
		return(-1);	/* shouldn't ever happen */
	}

	dir.home[0]=0;
	dir.dot[0]=0;
	dir.save[0]=0;
	dir.pic[0]=0;
	dir.demo[0]=0;
	dir.players[0]=0;
	dir.video[0]=0;

	strncpy(dir.home, home, HOMEPATHLEN);

	/* Construct "/home/dave/.quake2" */
	strncpy(dir.dot, dir.home, PATHLEN);
	strncat(dir.dot, "/"Q2DOT_DIR, PATHLEN);

	/* seed the others with the dot directory */
	strncat(dir.save, dir.dot, PATHLEN);
	strncat(dir.pic, dir.dot, PATHLEN);
	strncat(dir.demo, dir.dot, PATHLEN);
	strncat(dir.players, dir.dot, PATHLEN);
	strncat(dir.video, dir.dot, PATHLEN);

	/* put the last part on */
	strncat(dir.save, "/"SAVEDIR, PATHLEN);
	strncat(dir.pic, "/"PICDIR, PATHLEN);
	strncat(dir.demo, "/"DEMODIR, PATHLEN);
	strncat(dir.players, "/"PLAYERSDIR, PATHLEN);
	strncat(dir.video, "/"VIDEODIR, PATHLEN);

	/* the other exit shouldn't ever happen */
	return(1);
}

/************************************************************************/

int checkdirs(void)
{
	char	skindir[PATHLEN];
	char	skins[PATHLEN];
	char	videodir[PATHLEN];
	

	/* For some odd reason, I need to #include <sys/stat.h> to make */
	/* gcc shut up about mkdir(2) being implicitly declared here. */

	if (chdir(dir.dot) != 0) {
		if (errno == ENOENT) {
			printf("%s doesn't exist, making...\n", dir.dot);
			if (mkdir(dir.dot, DIRMODE) != 0) {
				perror("Problem making $HOME/.quake2\n");
				return(-1);
			}
			if (chown(dir.dot, getuid(), getgid()) != 0) {
				perror("Problem chowning $HOME/.quake2\n");
				return(-1);
			}			
		}
		else
			return(-1);
	}

	if (chdir(dir.save) != 0) {
		if (errno == ENOENT) {
			if (mkdir(dir.save, DIRMODE) != 0) {
				perror("Problem making the save directory.\n");
				return(-1);
			}
			if (chown(dir.save, getuid(), getgid()) != 0) {
				perror("Problem chowning the save directory.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	if (chdir(dir.pic) != 0) {
		if (errno == ENOENT) {
			if (mkdir(dir.pic, DIRMODE) != 0) {
				perror("Problem making the srnshots directory.\n");
				return(-1);
			}
			if (chown(dir.pic, getuid(), getgid()) != 0) {
				perror("Problem chowning the scrnshots directory.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	if (chdir(dir.demo) != 0) {
		if (errno == ENOENT) {
			if (mkdir(dir.demo, DIRMODE) != 0) {
				perror("Problem making the demo directory.\n");
				return(1);
			}
			if (chown(dir.demo, getuid(), getgid()) != 0) {
				perror("Problem chowning the demo directory.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	if (chdir(dir.players) != 0) {
		if (errno == ENOENT) {
			if (mkdir(dir.players, DIRMODE) != 0) {
				perror("Problem making the players directory.\n");
				return(-1);
			}
			if (chown(dir.players, getuid(), getgid()) != 0) {
				perror("Problem chowning the players directory.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	if (chdir(dir.video) != 0) {
		if (errno == ENOENT) {
			videodir[0] = 0;
			strncat(videodir, line.basedir, PATHLEN);
			strncat(videodir, SYS_BASE, PATHLEN);
			strcat(videodir, "/"VIDEODIR);
			if (symlink(videodir, dir.video) != 0) {
				perror("Problem symlinking to the video directory\n");
				return(-1);
			}
			if (chown(dir.video, getuid(), getgid()) != 0) {
				perror("Problem chowning the video directory symlink.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}	

	skins[0] = 0;
	strncat(skins, dir.players, PATHLEN);
	strncat(skins, "/male", PATHLEN);
	if (chdir(skins) != 0) {
		if (errno == ENOENT) {
			skindir[0] = 0;
			strncat(skindir, line.basedir, PATHLEN);
			strncat(skindir, SYS_BASE, PATHLEN);
			strncat(skindir, "/"PLAYERSDIR, PATHLEN);
			strncat(skindir, "/male", PATHLEN);
			if(symlink(skindir, skins) != 0) {
				perror("Problem symlinking to the male skins directory\n");
				return(-1);
			}
			if(chown(skins, getuid(), getgid()) != 0) {
				perror("Problem chowning male skins symlink.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	skins[0] = 0;
	strncat(skins, dir.players, PATHLEN);
	strncat(skins, "/female", PATHLEN);
	if (chdir(skins) != 0) {
		if (errno == ENOENT) {
			skindir[0] = 0;
			strncat(skindir, line.basedir, PATHLEN);
			strncat(skindir, SYS_BASE, PATHLEN);
			strncat(skindir, "/"PLAYERSDIR, PATHLEN);
			strncat(skindir, "/female", PATHLEN);
			if(symlink(skindir, skins) != 0) {
				perror("Problem symlinking to the female skins directory\n");
				return(-1);
			}
			if(chown(skins, getuid(), getgid()) != 0) {
				perror("Problem chowning female skins symlink.\n");
				return(-1);
			}
		}
		else
			return(-1);
	}

	return(1);
}

/************************************************************************/

int makelinks(void)
{
	char todir[PATHLEN+1];


	todir[0] = 0;
	strncat(todir, line.basedir, PATHLEN);
	strncat(todir, BASEQ2, PATHLEN);

	if (symlink(dir.dot, todir) != 0) {
		perror("Problem symlinking BASEDIR/baseq2 to $HOME/.quake2\n");
		return(-1);
	}
	return(1);
}

/************************************************************************/

int breaklinks(void)
{
	char todir[PATHLEN+1];

	todir[0] = 0;
	strncat(todir, line.basedir, PATHLEN);
	strncat(todir, BASEQ2, PATHLEN);

	if (unlink(todir) != 0) {
		perror("Problem removing symlink to $HOME/.quake2\n");
		return(-1);
	}
	return(1);
}

/************************************************************************/


